----------------------------------------------------------------------------------------------------------------------------------------------------
-- WhatRepRecipes - View vendor sell values for all crafted items and reputation requirements, source, price and mats for all unskilled recipe ranks
----------------------------------------------------------------------------------------------------------------------------------------------------
-- Hooks.lua - Hooking into TradeSkillFrame (DetailsFrame + RecipeList)
--
-- Author: Expelliarm5s / August 2022 / All Rights Reserved
--
-- Version 1.2.11
-------------------------------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI

-- luacheck: globals TradeSkillFrame TradeSkillFrameCloseButton
-- luacheck: globals Skillet SkilletExtraDetailTextLeft
-- luacheck: globals C_TradeSkillUI

local addonName, addon = ...
local Hooks = addon:NewModule("Hooks", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibQTip = LibStub("LibQTip-1.0")
local private = {}
-------------------------------------------------------------------------------------------------------

-- remember closed list per button push
private.listStayClosed = true

------------------------------------------------------------------------------
-- Debug Stuff

function Hooks:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("Hks~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Enable Hooks

function Hooks:Login()
	Hooks:DebugPrintf("Login()")

	if not addon.isClassic then
		Hooks:HookTradeSkillFrame("Login")
		-- soon
		-- Hooks:HookTSM("Login")
		-- we are using the debugLog environment
		addon.debuglog = addon.debuglog or {}
		Hooks:RegisterEvent("ADDON_LOADED", "OnAddonLoaded")
		Hooks:RegisterEvent("TRADE_SKILL_CLOSE", "TRADE_SKILL_CLOSE")
	end
	if Skillet then
		Hooks:HookSkillet("Login")
	end
end

function Hooks:OnAddonLoaded(_, aName)
	if aName == "Blizzard_TradeSkillUI" then
		Hooks:HookTradeSkillFrame("Delayed")
	end
	if aName == "Skillet" then
		Hooks:HookSkillet("Delayed")
	end
	-- soon
	--if aName == "TradeSkillMaster" then
	--	Hooks:HookTSM("Delayed")
	--end
end

function Hooks:TRADE_SKILL_SHOW(list)
	Hooks:DebugPrintf("OK~TRADE_SKILL_SHOW(%s)", tostring(list))

	local doUpdate = true
	if not addon.GUI.display and not private.listStayClosed and not addon.UIClosed then
		addon:ShowGUI(list)
		doUpdate = false
	end
	if addon.GUI.display then
		if list and list ~= addon.GUI.view then
			addon:ShowGUI(list)
			doUpdate = false
		end
		if doUpdate then
			Hooks:FillLogFromRecipes(addon.GUI.view)
			addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
		end
	end
	Hooks:CreateWRRButton()
end

function Hooks:TRADE_SKILL_CLOSE()
	Hooks:DebugPrintf("OK~TRADE_SKILL_CLOSE()")
	if addon.GUI.display then
		addon.GUI.display:Fire("OnClose")
	end
end

-- Place the WRR button on the trade skill frame
function Hooks:CreateWRRButton()
	Hooks:DebugPrintf("CreateWRRButton()")

	if private.wrrButton then
		Hooks:DebugPrintf("  already there")
		return
	end

	if TradeSkillFrame and TradeSkillFrameCloseButton and not private.wrrButton then
		private.wrrButton = CreateFrame("Button", nil, TradeSkillFrame, "UIPanelButtonTemplate")
		private.wrrButton:SetHeight(20)
		private.wrrButton:RegisterForClicks("LeftButtonUp")
		private.wrrButton:SetText("WRR")

		-- leftclick opens or closes list
		private.wrrButton:SetScript("OnClick", function(_, mouseButton, _)
			Hooks:DebugPrintf("CreateWRRButton_OnClick(%s)", tostring(mouseButton))
			if addon.GUI.display then
				addon.GUI.display:Hide()
				private.listStayClosed = true -- remember CLOSE button push this session
			else
				private.listStayClosed = false
				addon.UIClosed = false -- catch CLOSE button of WRRFrame widget
				local list
				if TradeSkillFrame and TradeSkillFrame.IsVisible and
					TradeSkillFrame:IsVisible() and TradeSkillFrame.TitleText and TradeSkillFrame.TitleText:GetText() then
					list = TradeSkillFrame.TitleText:GetText()
				end
				addon:ShowGUI(list)
			end
		end)

		-- mouseover shows usage tooltip
		private.wrrButton:SetScript("OnEnter", function(this)
			local tooltip = LibQTip:Acquire(addon)
			if tooltip then
				tooltip:ClearAllPoints()
				tooltip:SetClampedToScreen(true)
				tooltip:SetPoint("BOTTOMLEFT", this, 0, 20)
				tooltip:SetAutoHideDelay(0.1, this)
				tooltip:EnableMouse(true)
				tooltip:Clear()
				tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

				local line = tooltip:AddHeader()
				tooltip:SetCell(line, 1, "|cfffed100" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r", nil, "CENTER", 2)
				tooltip:AddSeparator()
				line = tooltip:AddLine()
				if addon.GUI.display then
					tooltip:SetCell(line, 1, L["Hide Recipe List"], 2)
				else
					tooltip:SetCell(line, 1, L["Show Recipe List"], 2)
				end

				tooltip:Show()
			end
		end)

		-- left of Trade Skill Link button
		private.wrrButton:SetPoint("RIGHT", TradeSkillFrameCloseButton, "LEFT", 6, 20)
		private.wrrButton:SetWidth(private.wrrButton:GetTextWidth() + 40)
	end
end

-- Place the WRR button on the TSM frame
function Hooks.CreateWRRButtonTSM(isVisible, tsmFrame)
	Hooks:DebugPrintf("CreateWRRButtonTSM()")

--	if wrrButton then
--		Hooks:DebugPrintf("  already there")
--		return
--	end

--	if TradeSkillFrame and TradeSkillFrameCloseButton and not wrrButton then
	local wrrButton = CreateFrame("Button", nil, tsmFrame, "UIPanelButtonTemplate")
		wrrButton:SetHeight(20)
		wrrButton:RegisterForClicks("LeftButtonUp")
		wrrButton:SetText("WRR")

		-- leftclick opens or closes list
		wrrButton:SetScript("OnClick", function(_, mouseButton, _)
			Hooks:DebugPrintf("CreateWRRButton_OnClick(%s)", tostring(mouseButton))
			if addon.GUI.display then
				addon.GUI.display:Hide()
				private.listStayClosed = true -- remember CLOSE button push this session
			else
				private.listStayClosed = false
				addon.UIClosed = false -- catch CLOSE button of WRRFrame widget
				local list
				if TradeSkillFrame and TradeSkillFrame.IsVisible and
					TradeSkillFrame:IsVisible() and TradeSkillFrame.TitleText and TradeSkillFrame.TitleText:GetText() then
					list = TradeSkillFrame.TitleText:GetText()
				end
				addon:ShowGUI(list)
			end
		end)

		-- mouseover shows usage tooltip
		wrrButton:SetScript("OnEnter", function(this)
			local tooltip = LibQTip:Acquire(addon)
			if tooltip then
				tooltip:ClearAllPoints()
				tooltip:SetClampedToScreen(true)
				tooltip:SetPoint("BOTTOMLEFT", this, 0, 20)
				tooltip:SetAutoHideDelay(0.1, this)
				tooltip:EnableMouse(true)
				tooltip:Clear()
				tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

				local line = tooltip:AddHeader()
				tooltip:SetCell(line, 1, "|cfffed100" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")|r", nil, "CENTER", 2)
				tooltip:AddSeparator()
				line = tooltip:AddLine()
				if addon.GUI.display then
					tooltip:SetCell(line, 1, L["Hide Recipe List"], 2)
				else
					tooltip:SetCell(line, 1, L["Show Recipe List"], 2)
				end

				tooltip:Show()
			end
		end)

		-- left of Trade Skill Link button
		wrrButton:SetPoint("RIGHT", tsmFrame, "TOPRIGHT", -63, 9)
		wrrButton:SetWidth(wrrButton:GetTextWidth() + 40)
	--end
end

-- Enable hook into TSM
function Hooks:HookTSM(mode)
	Hooks:DebugPrintf("HookTSM(%s)", tostring(mode))
	if TSM_API and TSM_API.RegisterUICallback then
		if not private.tsmHooked then
			Hooks:DebugPrintf("  register with TSM_API.RegisterUICallback")
			private.tsmHooked = true
			TSM_API.RegisterUICallback("CRAFTING", "WRR", Hooks.CreateWRRButtonTSM)
		end
	end
end

-- Enable hook into TradeSkillFrame.DetailsFrame:RefreshDisplay() and TradeSkillFrame.RecipeList:RebuildDataList()
function Hooks:HookTradeSkillFrame(mode)
	Hooks:DebugPrintf("HookTradeSkillFrame(%s)", tostring(mode))
	if TradeSkillFrame then
		if TradeSkillFrame.DetailsFrame then
			if not Hooks:IsHooked(TradeSkillFrame.DetailsFrame, "RefreshDisplay") then
				Hooks:DebugPrintf("  TradeSkillFrame.DetailsFrame is now hooked.")
				Hooks:SecureHook(TradeSkillFrame.DetailsFrame, "RefreshDisplay")
			else
				Hooks:DebugPrintf("ERR~ already hooked TradeSkillFrame.DetailsFrame.RefreshDisplay")
			end
		else
			Hooks:DebugPrintf("ERR~ no TradeSkillFrame.DetailsFrame")
		end
		if TradeSkillFrame.RecipeList then
			if not Hooks:IsHooked(TradeSkillFrame.RecipeList, "RebuildDataList") then
				Hooks:DebugPrintf("  TradeSkillFrame.RecipeList.RebuildDataList is now hooked.")
				Hooks:SecureHook(TradeSkillFrame.RecipeList, "RebuildDataList")
			else
				Hooks:DebugPrintf("ERR~ already hooked TradeSkillFrame.RecipeList.RebuildDataList")
			end
		end
	else
		Hooks:DebugPrintf("ERR~ no TradeSkillFrame")
	end
end

-- Enable hook into Skillet.UpdateDetailsWindow()
function Hooks:HookSkillet(mode)
	Hooks:DebugPrintf("HookSkillet(%s)", tostring(mode))
	if Skillet then
		if Skillet.UpdateDetailsWindow then
			if not Hooks:IsHooked(Skillet, "UpdateDetailsWindow") then
				Hooks:DebugPrintf("  Skillet.UpdateDetailsWindow is now hooked.")
				Hooks:SecureHook(Skillet, "UpdateDetailsWindow")
				if Skillet.db and Skillet.db.profile then
					Skillet.db.profile.show_recipe_source_for_learned = true
				end
			else
				Hooks:DebugPrintf("ERR~ already hooked Skillet.UpdateDetailsWindow")
			end
		else
			Hooks:DebugPrintf("ERR~ no Skillet.UpdateDetailsWindow")
		end
	else
		Hooks:DebugPrintf("ERR~ no Skillet")
	end
end

-- Extend Blizzards TradeSkillFrame.RecipeList:RebuildDataList()
function Hooks:RebuildDataList(this)
	Hooks:DebugPrintf("RebuildDataList(%s)", tostring(this))

	local list
	if TradeSkillFrame and TradeSkillFrame.TitleText and TradeSkillFrame.TitleText:GetText() then
		list = TradeSkillFrame.TitleText:GetText()
	end

	if not list then
		Hooks:DebugPrintf("ERR~  missing TradeSkillFrame name!")
		return
	end

	Hooks:TRADE_SKILL_SHOW(list)
end

function Hooks:SetRebuildUI()
	private.rebuildUI = true
end

function Hooks:GetTradeSkill()
	Hooks:DebugPrintf("GetTradeSkill()")
	local list
	if TradeSkillFrame and TradeSkillFrame.IsVisible and
		TradeSkillFrame:IsVisible() and TradeSkillFrame.TitleText and TradeSkillFrame.TitleText:GetText() then
		list = TradeSkillFrame.TitleText:GetText()
	end
	Hooks:DebugPrintf("  = %s", tostring(list))
	return list
end

-- Create Tab, Fill Data, Show UI
function Hooks:Load(tab)
	Hooks:DebugPrintf("Load(%s)", tostring(tab))

	local list
	if TradeSkillFrame and TradeSkillFrame.IsVisible and
		TradeSkillFrame:IsVisible() and TradeSkillFrame.TitleText and TradeSkillFrame.TitleText:GetText() then
		list = TradeSkillFrame.TitleText:GetText()
	end

	if not tab then
		if not list then
			Hooks:DebugPrintf("ERR~  tab is nil")
			return
		else
			Hooks:DebugPrintf("  use list %s", tostring(list))
		end
	end

	local theTab = tab or list

	-- create Log/Tab if not present
	if not addon.debuglog[theTab] or private.rebuildUI then
		Hooks:DebugPrintf("  no log %s", tostring(theTab))

		-- flag tab for reload
		if private.rebuildUI then
			Hooks:DebugPrintf("  rebuildUI for %s: #addon.GUI.guiTabs.order = %s", tostring(theTab), tostring(#addon.GUI.guiTabs.order))
			local thisName
			for i, name in ipairs(addon.GUI.guiTabs.order) do
				Hooks:DebugPrintf("  rebuildUI for %s: i=%s, name=%s", tostring(theTab), tostring(i), tostring(name))
				if name and name == theTab then
					thisName = i
				end
			end
			if thisName then
				Hooks:DebugPrintf("  rebuildUI for %s: tab num %s", tostring(theTab), tostring(thisName))
				table.remove(addon.GUI.guiTabs.order, thisName)
			else
				Hooks:DebugPrintf("ERR~  rebuildUI for %s: no tab num found!", tostring(theTab))
			end
		end

		addon.GUI.CreateRecipesTab(theTab, nil, private.rebuildUI)
	else
		-- to show this tab in GUI:Load()
		addon.GUI.guiTabs.lastSelected = theTab
		Hooks:DebugPrintf("  log %s", tostring(theTab))
	end

	private.rebuildUI = false

	-- put relevant recipes into the log
	Hooks:FillLogFromRecipes(theTab)

	-- show the GUI
	Hooks:DebugPrintf("  -> GUI:Load(%s)", tostring(theTab))
	addon.GUI:Load(theTab)
	if addon.GUI.debugLog_colCtrl[theTab] and addon.GUI.debugLog_colCtrl[theTab].w then
		addon.GUI.debugLog_lastSortTitle[theTab] = addon.GUI.debugLog_colCtrl[theTab].w:GetSortTitle()
		-- addon:Printf("set GUI.debugLog_lastSortTitle[%s] = %s", tostring(theTab), tostring(addon.GUI.debugLog_lastSortTitle[theTab]))
	end
	Hooks:DebugPrintf("  <<Load(%s)", tostring(theTab))
end

-- Create Log with Recipes
function Hooks:FillLogFromRecipes(tab)
	Hooks:DebugPrintf("FillLogFromRecipes(%s)", tostring(tab))

	if not tab then
		Hooks:DebugPrintf("ERR~  tab is nil")
		return
	end

	addon.GUI.DebugLog_Reset(tab)

	if not TradeSkillFrame or not TradeSkillFrame.RecipeList then
		Hooks:DebugPrintf("ERR~  no TradeSkillFrame.RecipeList")
		return
	end

	-- if there are no real recipes, don't open list
	local recipesPresent = false
	if TradeSkillFrame.RecipeList.dataList then
		for _, listData in ipairs(TradeSkillFrame.RecipeList.dataList) do
			if listData.type and listData.type == "recipe" then
				recipesPresent = true
			end
		end
	end

	local valid = true
	if not recipesPresent then
		Hooks:DebugPrintf("  -> no recipes!")
		valid = false
	end

	-- fill recipe list
	-- local modified = false
	if TradeSkillFrame.RecipeList.dataList then
		for i, listData in ipairs(TradeSkillFrame.RecipeList.dataList) do
			if listData.type and listData.type == "recipe" then
				if C_TradeSkillUI.GetOptionalReagentInfo and C_TradeSkillUI.GetOptionalReagentInfo(listData.recipeID)
					and C_TradeSkillUI.GetOptionalReagentInfo(listData.recipeID)[1] then

					Hooks:DebugPrintf("8~  %s. %s has optional reagents", tostring(i), tostring(listData.name))

					-- for Relic recipes, mark name with '+'
					if listData.name and type(listData.name) == "string" then
						if not listData.name:match("^+") then
							-- modified = true
							listData.name = "+" .. listData.name
						end
					end

					if valid then
						addon.Item:SellsAddRecipe(TradeSkillFrame.RecipeList, tab, listData.recipeID, listData.name)
					end
				else
					Hooks:DebugPrintf("8~  %s. %s has no optional reagents", tostring(i), tostring(listData.name))

					if valid then
						addon.Item:SellsAddRecipe(TradeSkillFrame.RecipeList, tab, listData.recipeID, listData.name)
					end
				end
			end
		end
	end
end

------------------------------------------------------------------------------
-- Extend infos in the recipe details windows

-- Add addon name to separate game infos from made up infos
function Hooks:GetHeaderText(sourceText)
	return "|n|n|cFF33FF99" .. addonName .. " (" ..  addon.METADATA.VERSION .. "):|r" .. tostring(sourceText) .. "|n|n"
end

-- Extend Blizzards TradeSkillFrame.DetailsFrame:RefreshDisplay()
function Hooks:RefreshDisplay(this)
	Hooks:DebugPrintf("RefreshDisplay(%s)", tostring(this))

	if C_TradeSkillUI.IsRecipeSearchInProgress() or not C_TradeSkillUI.IsTradeSkillReady() or C_TradeSkillUI.IsNPCCrafting() then
		return
	end

	local recipeInfo = this.selectedRecipeID and C_TradeSkillUI.GetRecipeInfo(this.selectedRecipeID);

	if recipeInfo and recipeInfo.recipeID and this.Contents and this.Contents.SourceText then
		local oldSourceText = this.Contents.SourceText:GetText()
		local sourceText = addon.Item:GetRecipeSourceText(recipeInfo)

		if sourceText ~= "" then
			sourceText = (oldSourceText or "") .. Hooks:GetHeaderText(sourceText)

			if oldSourceText then
				this.Contents.SourceText:SetText(sourceText)
			else
				this:AddContentWidget(this.Contents.SourceText);
				this.Contents.SourceText:SetText(sourceText)

				if this.optionalReagentSlots and this.Contents.OptionalReagentLabel and this.Contents.OptionalReagentLabel:IsShown() then
					local info = C_TradeSkillUI.GetOptionalReagentInfo(this.selectedRecipeID)
					if info and type(info) == "table" and #info > 2 then
						this.Contents.SourceText:SetPoint("TOP", this.Contents.OptionalReagentLabel, "BOTTOM", 0, -70);
					else
						this.Contents.SourceText:SetPoint("TOP", this.Contents.OptionalReagentLabel, "BOTTOM", 0, -35);
					end
				else
					local numReagents = C_TradeSkillUI.GetRecipeNumReagents(this.selectedRecipeID)
					if numReagents > 0 then
						this.Contents.SourceText:SetPoint("TOP", this.Contents.Reagents[numReagents], "BOTTOM", 0, -10);
					else
						this.Contents.SourceText:SetPoint("TOP", this.Contents.ReagentLabel, "TOP");
					end
				end

				this.Contents.SourceText:Show()
			end
		end
	end
end

-- Extend Skillet:UpdateDetailsWindow()
function Hooks:UpdateDetailsWindow(this, skillIndex)
	Hooks:DebugPrintf("UpdateDetailsWindow(%s, %s)", tostring(this), tostring(skillIndex))

	if skillIndex and skillIndex > 0 then
		Hooks:DebugPrintf("UpdateDetailsWindow, 1")
		if this.GetSkill and this.currentPlayer and this.currentTrade then
			Hooks:DebugPrintf("UpdateDetailsWindow, 2")
			local skill = this:GetSkill(this.currentPlayer, this.currentTrade, skillIndex)

			if skill and skill.id then
				Hooks:DebugPrintf(" skill.id %s", skill.id or "?")

				local recipeInfo = skill.id and C_TradeSkillUI.GetRecipeInfo(skill.id);
				if recipeInfo and recipeInfo.recipeID and SkilletExtraDetailTextLeft then

					local oldSourceText = SkilletExtraDetailTextLeft:GetText()
					local sourceText = addon.Item:GetRecipeSourceText(recipeInfo)

					if oldSourceText then
						if sourceText ~= "" then
							sourceText = (oldSourceText or "") .. Hooks:GetHeaderText(sourceText)
							SkilletExtraDetailTextLeft:SetText(sourceText)
						end
					end
				end
			end
		end
	end
end

-- EOF
