----------------------------------------------------------------------------------------------------------------------------------------------------
-- WhatRepRecipes - View vendor sell values for all crafted items and reputation requirements, source, price and mats for all unskilled recipe ranks
----------------------------------------------------------------------------------------------------------------------------------------------------
-- Modules/Options.lua - Addon Options
--
-- Author: Expelliarm5s / August 2022 / All Rights Reserved
--
-- Version 1.2.11
-------------------------------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI

local addonName, addon = ...
local Options = addon:NewModule("Options", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibDBIcon = LibStub("LibDBIcon-1.0")
--------------------------------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Settings

Options.defaults = {
	profile = {
	},
	global = {
		timeFormat = "default",
		maxVerbosity = "6",
		showExport = true,
		discardCopper = false,
		discardSilver = false,
		vendorSellSort = true,
		textSize = "12",
		isExp = false,
		showMinimapButton = true,
	},
}

Options.timeFormat = {}
Options.timeFormat["ago"]  = L["_ Hr. _ Min. ago"]
Options.timeFormat["%d.%m %H:%M:%S"] = "DD.MM HH:MM"
Options.timeFormat["%m/%d %H:%M:%S"] = "MM/DD HH:MM"
Options.timeFormat["%m-%d %H:%M:%S"] = "MM-DD HH:MM"

Options.textSizes = {}
Options.textSizes["10"]  = "10"
Options.textSizes["11"]  = "11"
Options.textSizes["12"]  = "12"
Options.textSizes["13"]  = "13"

------------------------------------------------------------------------------
-- Debug Stuff

function Options:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("Opt~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Addon Loading / Player Login/Logout

function Options.GetOptions(_, _, appName)
	if appName == addonName then

		local wowV, wowP = GetBuildInfo()
		local wowVersion = "|nGame: WoW, Flavor: " .. (addon.isClassic and "Vanilla/TBC/Wrath" or "Mainline") .. ", Version: " .. wowV .. ", Build: " .. wowP

		local options = {
			type = "group",
			name = addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")",
			get = function(info)
					return addon.db.global[info[#info]] or ""
				end,
			set = function(info, value)
					addon.db.global[info[#info]] = value
					Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
					if addon.GUI.display then
						addon.Hooks:FillLogFromRecipes(addon.GUI.view)
						addon.GUI.DebugLog_UpdateData(addon.GUI.view, true)
						-- addon.GUI.container:Reload()
					end
				end,
			args = {
				desc0 = {
					type = "description",
					order = 0,
					name = "|cff99ccff-: by " .. GetAddOnMetadata(addonName, "Author") .. " :-|r|n|n" .. GetAddOnMetadata(addonName, "Notes"),
					fontSize = "medium",
				},
				desc001 = {
					type = "description",
					order = 0.01,
					name = wowVersion,
				},
				header100 = {
					type = "header",
					name = L["UI"],
					order = 1.00,
				},
				showMinimapButton = {
					type = "toggle",
					name = L["Show Minimap Button"],
					desc = L["If checked, the minimap button is present."],
					order = 1.001,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
					set = function(info, value)
							addon.db.global[info[#info]] = value
							if value then
								LibDBIcon:Show(addonName)
							else
								LibDBIcon:Hide(addonName)
							end
							Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
						end,
				},
				desc102 = {
					type = "description",
					order = 1.002,
					name = L["|cffff8888Changing the font size causes an user interface reload!|r"],
					fontSize = "medium",
				},
				textSize = {
					type = "select",
					style = "dropdown",
					name = L["Font Size"],
					desc = L["Choose the font size. Changing the size causes an user interface reload!"],
					order = 1.003,
					width = "double",
					values = Options.textSizes,
					set = function(info, value)
						addon.db.global[info[#info]] = value
						Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
						ReloadUI()
					end
				},
				header01a = {
					type = "header",
					name = L["Recipe List"],
					order = 2.00,
				},
				vendorSellSort = {
					type = "toggle",
					name = L["Sort by VendorSell"],
					desc = L["If checked, table is sorted by VendorSell."],
					order = 2.2,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
					set = function(info, value)
						addon.db.global[info[#info]] = value
						Options:DebugPrintf("OK~Set %s = %s", tostring(info[#info]), tostring(value))
						if value then
							addon.GUI.debugLog_defaultSortTitle = -10
						else
							addon.GUI.debugLog_defaultSortTitle = 1
						end
						if addon.GUI.display then
							if addon.GUI.view and addon.GUI.debugLog_lastSortTitle[addon.GUI.view] then
								addon.GUI.debugLog_lastSortTitle[addon.GUI.view] = nil
							end
							addon.GUI.container:Reload()
						end
					end
				},
				discardSilver = {
					type = "toggle",
					name = L["Discard Silver"],
					desc = L["If checked, any values below 1g are not displayed."],
					order = 2.3,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				discardCopper = {
					type = "toggle",
					name = L["Discard Copper"],
					desc = L["If checked, any values below 1s are not displayed."],
					order = 2.4,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
			},
		}

		return options
	end
end

-- EOF
